@@ -23,19 +23,32 @@ package Util;
 use strict;
 
 use Exporter;
-our @ISA = qw(Exporter);
-our @EXPORT = qw(parse_dpkg_control
+
+# Force export as soon as possible, since some of the modules we load also
+# depend on us and the sequencing can cause things not to be exported
+# otherwise.
+our (@ISA, @EXPORT);
+BEGIN {
+    @ISA = qw(Exporter);
+    @EXPORT = qw(parse_dpkg_control
 	read_dpkg_control
 	get_deb_info
 	get_dsc_info
 	slurp_entire_file
-	get_file_md5
+	get_file_checksum
 	file_is_encoded_in_non_utf8
 	fail
-	system_env);
+	system_env
+	delete_dir
+	copy_dir
+	gunzip_file
+	touch_file
+	perm2oct);
+}
 
 use FileHandle;
-use Pipeline;
+use Lintian::Command qw(spawn);
+use Lintian::Output qw(string);
 use Digest::MD5;
 
 # general function to read dpkg control files
@@ -56,6 +69,7 @@ sub parse_dpkg_control {
     my $open_section = 0;
     my $last_tag;
 
+    local $_;
     while (<$CONTROL>) {
 	chomp;
 
@@ -120,7 +134,7 @@ sub read_dpkg_control {
     }
 
     my $CONTROL = FileHandle->new;
-    open($CONTROL,$file)
+    open($CONTROL, '<', $file)
 	or fail("cannot open control file $file for reading: $!");
     my @data = parse_dpkg_control($CONTROL, $debconf_flag);
     close($CONTROL)
@@ -136,13 +150,13 @@ sub get_deb_info {
     }
 
     # `dpkg-deb -f $file' is very slow. Instead, we use ar and tar.
-    my $CONTROL = FileHandle->new;
-    pipeline_open($CONTROL,
-		  (sub { exec 'ar', 'p', $file, 'control.tar.gz' }),
-		  (sub { exec 'tar', '--wildcards', '-xzO', '-f', '-', '*control' }))
-	or fail("cannot fork to unpack $file: $!\n");
-    my @data = parse_dpkg_control($CONTROL);
-    close($CONTROL) or fail("broken input pipe for unpacking $file: $!");
+    my $opts = { pipe_out => FileHandle->new };
+    spawn($opts,
+	  ['ar', 'p', $file, 'control.tar.gz'],
+	  '|', ['tar', '--wildcards', '-xzO', '-f', '-', '*control'])
+	or fail("cannot fork to unpack $file: $opts->{exception}\n");
+    my @data = parse_dpkg_control($opts->{pipe_out});
+    $opts->{harness}->finish();
     return $data[0];
 }
 
@@ -169,30 +183,34 @@ sub _ensure_file_is_sane {
 
 sub slurp_entire_file {
     my $file = shift;
-    open(C,$file)
+    open(C, '<', $file)
 	or fail("cannot open file $file for reading: $!");
-    my $save = $/;
-    undef $/;
+    local $/;
     local $_ = <C>;
-    $/ = $save;
     close(C);
     return $_;
 }
 
-sub get_file_md5 {
-	my $file = shift;
-	open FILE, $file or fail("Couldn't open $file");
-	my $md5 = Digest::MD5->new;
-	$md5->addfile(*FILE);
+sub get_file_checksum {
+	my ($alg, $file) = @_;
+	open (FILE, '<', $file) or fail("Couldn't open $file");
+	my $digest;
+	if ($alg eq 'md5') {
+	    $digest = Digest::MD5->new;
+	} elsif ($alg =~ /sha(\d+)/) {
+	    require Digest::SHA;
+	    $digest = Digest::SHA->new($1);
+	}
+	$digest->addfile(*FILE);
 	close FILE or fail("Couldn't close $file");
-	return $md5->hexdigest;
+	return $digest->hexdigest;
 }
 
 sub file_is_encoded_in_non_utf8 {
 	my ($file, $type, $pkg) = @_;
 	my $non_utf8 = 0;
-	
-	open ICONV, "env LANG=C iconv -f utf8 -t utf8 $file 2>&1 |"
+
+	open (ICONV, '-|', "env LANG=C iconv -f utf8 -t utf8 $file 2>&1")
 	    or fail("failure while checking encoding of $file for $type package $pkg");
 	my $line = 1;
 	while (<ICONV>) {
@@ -225,20 +243,73 @@ sub system_env {
     }
 }
 
-# ------------------------
+# Translate permission strings like `-rwxrwxrwx' into an octal number.
+sub perm2oct {
+    my ($t) = @_;
+
+    my $o = 0;
+
+    $t =~ m/^.(.)(.)(.)(.)(.)(.)(.)(.)(.)/o;
+
+    $o += 00400 if $1 eq 'r';	# owner read
+    $o += 00200 if $2 eq 'w';	# owner write
+    $o += 00100 if $3 eq 'x';	# owner execute
+    $o += 04000 if $3 eq 'S';	# setuid
+    $o += 04100 if $3 eq 's';	# setuid + owner execute
+    $o += 00040 if $4 eq 'r';	# group read
+    $o += 00020 if $5 eq 'w';	# group write
+    $o += 00010 if $6 eq 'x';	# group execute
+    $o += 02000 if $6 eq 'S';	# setgid
+    $o += 02010 if $6 eq 's';	# setgid + group execute
+    $o += 00004 if $7 eq 'r';	# other read
+    $o += 00002 if $8 eq 'w';	# other write
+    $o += 00001 if $9 eq 'x';	# other execute
+    $o += 01000 if $9 eq 'T';	# stickybit
+    $o += 01001 if $9 eq 't';	# stickybit + other execute
+
+    return $o;
+}
+
+sub delete_dir {
+    return spawn(undef, ['rm', '-rf', '--', @_]);
+}
+
+sub copy_dir {
+    return spawn(undef, ['cp', '-a', '--', @_]);
+}
+
+sub gunzip_file {
+    my ($in, $out) = @_;
+    spawn({out => $out, fail => 'error'},
+	  ['gzip', '-dc', $in]);
+}
+
+# create an empty file
+# --okay, okay, this is not exactly what `touch' does :-)
+sub touch_file {
+    open(T, '>', $_[0]) or return 0;
+    close(T) or return 0;
+
+    return 1;
+}
 
 sub fail {
-    my $str = "internal error";
+    my $str;
     if (@_) {
-	$str .= ": ".join( "\n", @_)."\n";
+	$str = string('internal error', @_);
     } elsif ($!) {
-	$str .= ": $!\n";
+	$str = string('internal error', "$!");
     } else {
-	$str .= ".\n";
+	$str = string('internal error');
     }
     $! = 2; # set return code outside eval()
     die $str;
-
 }
 
 1;
+
+# Local Variables:
+# indent-tabs-mode: t
+# cperl-indent-level: 4
+# End:
+# vim: syntax=perl sw=4 ts=8
